__author__ = "Swas.py"
__title__ = "vscode"
__license__ = "MIT"
__copyright__ = "Copyright 2021 Swas.py"
__version__ = "1.4.6"

from . import window
from .compiler import build
from .extension import Extension
from .envMethods import env
from . import _types as ext
from ._types import *
